/*************************************************************************//**
 *
 * @file mpu6050.c
 * @brief Implementation of Cox based I2C Driver for Invensense MPU-6050
 * @version 1.1
 * @date 03/13/2013
 * @author Amo Xu  amo@amoxu.com
 * @copyright Copyright (c)  2012, Amo Xu   All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * * Neither the name of the <ORGANIZATION> nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#include "hard_com.c"
#include "mpu6050.h"
#include "hw_mpu6050.h"
#include <math.h>

#if CALIBRATE_AFTER_INIT
static long GYRO_X_SMPL_SUM;
static long GYRO_Y_SMPL_SUM;
static long GYRO_Z_SMPL_SUM;
#endif
/**
 * @brief Write 1 Byte of data into to specified register of sensor
 * @param rgAdd the register address of the sensor
 * @param rgData the data which will be write to sensor
 */
void MPU6050_WriteData(unsigned char rgAdd, unsigned char rgData) {
MPU_Write(MPU_Addr,rgAdd,rgData, MPU_I2C);
}

#if CALIBRATE_AFTER_INIT
/**
 * @brief Calibrate the sensor by collecting specified times data and get the
 * average data.
 */
void MPU6050_Calibrate(void) {
    int x = 0;
    for (x = 0; x < CALIBRATE_SAMPLES; x++) {
        GYRO_X_SMPL_SUM += MPU6050_ReadSensorData(GYRO_X);
        GYRO_Y_SMPL_SUM += MPU6050_ReadSensorData(GYRO_Y);
        GYRO_Z_SMPL_SUM += MPU6050_ReadSensorData(GYRO_Z);
    }

    GYRO_X_OFFSET = GYRO_X_SMPL_SUM / CALIBRATE_SAMPLES;
    GYRO_Y_OFFSET = GYRO_Y_SMPL_SUM / CALIBRATE_SAMPLES;
    GYRO_Z_OFFSET = GYRO_Z_SMPL_SUM / CALIBRATE_SAMPLES;
}
#endif

/**
 * @brief Write the configured settings into sensor and let it start to work
 */
void MPU6050_Config_Register(void) {
    MPU6050_WriteData(SMPLRT_DIV, MPU6050_SAMPLE_RATE_DIV);
    MPU6050_WriteData(CONFIG, 0x07 & MPU6050_DLPF_CONFIG);
    MPU6050_WriteData(GYRO_CONFIG, 0x18 & MPU6050_GYRO_FULL_SCALE);
    MPU6050_WriteData(ACCEL_CONFIG, 0x18 & MPU6050_ACCEL_FULL_SCALE);
    MPU6050_WriteData(PWR_MGMT_1, 0);
}

/**
 * @brief Configure the I2C Bus
 */
void MPU6050_Config_Peripherals(void) {
    	I2C_Init(MPU_I2C_Clock_Rate,MPU_I2C);
}

/**
 * @brief Initialize the MPU-6050 Device.
 *
 * @note MPU6050_Calibrate is switched by @ref CALIBRATE_AFTER_INIT Macro
 *
 */
void MPU6050_Init(void) {
    MPU6050_Config_Peripherals();
    MPU6050_Config_Register();

#if CALIBRATE_AFTER_INIT
    MPU6050_Calibrate();
#endif

}

/**
 * @brief Read 2 Bytes Data from specified Register Address of sensor
 * @param rgAdd is the address of register which need to be read
 * @return 16bit signed data
 */
short MPU6050_ReadSensorData(unsigned char rgAdd) {
    unsigned char ucBuffer[2];
    MPU_Read(MPU_Addr,rgAdd,2 ,ucBuffer,MPU_I2C );
    return (ucBuffer[0] << 8) | ucBuffer[1];
}

/**
 * @brief Read All Sensor Data and put them into a strutted data.
 * @return @ref MPU6050Data
 */
MPU6050Data MPU6050_ReadAllSensorData(void) {
    MPU6050Data gyroData;

    gyroData.Gyro_X = MPU6050_ReadSensorData(GYRO_X) - GYRO_X_OFFSET;
    gyroData.Gyro_Y = MPU6050_ReadSensorData(GYRO_Y) - GYRO_Y_OFFSET;
    gyroData.Gyro_Z = MPU6050_ReadSensorData(GYRO_Z) - GYRO_Z_OFFSET;
    gyroData.Accel_X = MPU6050_ReadSensorData(ACCEL_X);
    gyroData.Accel_Y = MPU6050_ReadSensorData(ACCEL_Y);
    gyroData.Accel_Z = MPU6050_ReadSensorData(ACCEL_Z);
    gyroData.Temp = MPU6050_ReadSensorData(TEMP);

    return gyroData;
}

void IMU_Poll(IMU_DATA *data)
{
	 static float pitch_n , pitch,pitch_w;
	  static float yaw_n , yaw,yaw_w;
	  static float roll_n , roll,roll_w;
	  static MPU6050Data MPU_Data;
	
	  pitch_n=(acos((float)MPU_Data.Accel_Z/sqrt(MPU_Data.Accel_X*MPU_Data.Accel_X+MPU_Data.Accel_Z*MPU_Data.Accel_Z))-1.58925 )*57.297469;// Accelerator degree
	  pitch_w=( ((float)(MPU_Data.Gyro_Y)) / 131.07  );  // angular speed calculation
	  pitch=0.98*(pitch+pitch_w*0.01)+0.02*pitch_n ;       // complementry filter
	  data->pitch=pitch;
	
	  roll_n=(acos((float)MPU_Data.Accel_Y/sqrt(MPU_Data.Accel_Y*MPU_Data.Accel_Y+MPU_Data.Accel_Z*MPU_Data.Accel_Z))-1.58925 )*57.297469;// Accelerator degree
	  roll_w=( ((float)(MPU_Data.Gyro_X)) / 131.07  );  // angular speed calculation
	  roll=0.98*(pitch+roll_w*0.01)+0.02*roll_n ;       // complementry filter
	  data->roll=roll;
	
	  yaw_n=(acos((float)MPU_Data.Accel_X/sqrt(MPU_Data.Accel_X*MPU_Data.Accel_X+MPU_Data.Accel_Y*MPU_Data.Accel_Y))-1.58925 )*57.297469;// Accelerator degree
	  yaw_w=( ((float)(MPU_Data.Gyro_Z)) / 131.07  );  // angular speed calculation
	  yaw=0.98*(yaw+yaw_w*0.01)+0.02*yaw_n ;       // complementry filter
	  data->yaw=yaw;
}
