/*************************************************************************//**
 *
 * @file mpu6050.h
 * @brief Header File of Cox based I2C Driver for Invensense MPU-6050
 * @version 1.1
 * @date 03/13/2013
 * @author Amo Xu  amo@amoxu.com
 * @copyright Copyright (c)  2012, Amo Xu   All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * * Neither the name of the <ORGANIZATION> nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#ifndef __MPU_6050_H
#define __MPU_6050_H

/**
 * If building with a C++ compiler, make all of the definitions in this header
 * have a C binding.
 */
 typedef struct{
	 float pitch;
	 float yaw;
	 float roll;
 } IMU_DATA;
typedef struct {
    int Gyro_X;
    int Gyro_Y;
    int Gyro_Z;
    int Accel_X;
    int Accel_Y;
    int Accel_Z;
    int Temp;
} MPU6050Data;

//! @addtogroup USER_CONF User Configurations
//! @brief Sensor parameters(scale, sample rate) configuration
//! @{

/**
 * Here indicate whether need to automatic calibration after initialize.
 * please set to 0 if you don't need automatic calibration
 */
#define CALIBRATE_AFTER_INIT 1

#if CALIBRATE_AFTER_INIT

/**
 * The quantity of samples which will be collected to be used for calibration after init
 */
#define CALIBRATE_SAMPLES 100
#endif


/**
 * @brief Configure the Sample Rate Divider
 * @param value from 0x00 to 0xff
 *
 * This register specifies the divider from the gyroscope output rate used to
 * generate the Sample Rate for the MPU-60X0.
 *
 * The sensor register output, FIFO output, DMP sampling and Motion detection are
 * all based on the Sample Rate.
 *
 * The Sample Rate is generated by dividing the gyroscope output rate by SMPLRT_DIV:
 * > Sample Rate = Gyroscope Output Rate / (1 + SMPLRT_DIV)
 *
 * where Gyroscope Output Rate = 8kHz when the DLPF is disabled (DLPF_CFG = 0 or 7), and 1kHz
 * when the DLPF is enabled (see Register 26).
 *
 * @note The accelerometer output rate is 1kHz.  This means that for a Sample Rate greater
 * than 1kHz, the same accelerometer sample may be output to the FIFO, DMP, and sensor
 * registers more than once.
 */
#define MPU6050_SAMPLE_RATE_DIV 0x00

/**
 * @brief Configure the full scale range of accelerometer outputs
 *
 * Select the full scale range of accelerometer outputs according to the following table.
 * AFS_SEL  | Full Scale Range
 * :--:     | :--:
 * @b 0     | +-2 g
 * @b 1     | +-4 g
 * @b 2     | +-8 g
 * @b 3     | +-16 g
 *
 * returned result is a 16bit signed integer, so if you want to get the gravity (g).
 * the result need to be divided by 65536/full scale range.\n
 * for example:
 * >     MPU6050_ACCEL_FULL_SCALE set to 2 (+-8 g)\n
 * >     65536/16.0 = 4096.0 \n
 * >     the read Accel_X result is 3863\n
 * >     which means the angle velocity is 3863/4096.0 ≈ 0.943 g
 */
#define MPU6050_ACCEL_FULL_SCALE 0

/**
 * @brief Configure the full scale range of the gyroscope outputs
 *
 * Select the full scale range of gyroscope outputs according to the following table.
 * FS_SEL   | Full Scale Range
 * :--:     | :--:
 * @b 0     | +-250 degree/s
 * @b 1     | +-500 degree/s
 * @b 2     | +-1000 degree/s
 * @b 3     | +-2000 degree/s
 *
 * returned result is a 16bit signed integer, so if you want to get the actual gyroscope angle velocity(degree/s).
 * the result need to be divided by 65536/full scale range.\n
 * for example:
 * >     MPU6050_GYRO_FULL_SCALE set to 1 (+-500 degree/s)\n
 * >     65536/1000 ≈ 66\n
 * >     the read Gyro_X result is 3863\n
 * >     which means the angle velocity is 3863/66 ≈ 59 degree/s
 */
#define MPU6050_GYRO_FULL_SCALE 1

/**
 * @brief The Digital Low Pass Filter configuration
 *
 * For configuration value selection, please refer to below table:
 * DLPF_CFG | Accel Bandwidth\n(Hz) | Accel Delay\n(ms) | Gyro Bandwidth\n(Hz) | Gyro Delay\n(ms) | Fs(kHz)
 * :--:     | :--:  | :--:  | :--:  | :--:  | :--:
 * @b 0     | 260   | 0     | 256   | 0.98  | 8
 * @b 1     | 184   | 2.0   | 188   | 1.9   | 1
 * @b 2     | 94    | 3.0   | 98    | 2.8   | 1
 * @b 3     | 44    | 4.9   | 42    | 4.8   | 1
 * @b 4     | 21    | 8.5   | 20    | 8.3   | 1
 * @b 5     | 10    | 13.8  | 10    | 13.4  | 1
 * @b 6     | 5     | 19.0  | 5     | 18.6  | 1
 * @b 7     | RESERVED | RESERVED | RESERVED | RESERVED | 8
 *
 */
#define MPU6050_DLPF_CONFIG 3


/**
 * @brief Slave Address of the device
 * The Slave Address of MPU-60X0 is 0b110100X which is 7 bits long,
 * THe LSB bit of the 7 bit address is determined by the logic level on pin AD0.
 * this allows 2 MPU-60X0 to be connected to the same I2C bus.
 *
 * Bit7 | Bit6  | Bit5  | Bit4  | Bit3  | Bit2  | Bit1  | Bit0  |
 * :--: | :--:  | :--:  | :--:  | :--:  | :--:  | :--:  | :--:  |
 * 0    | 1     | 1     | 0     | 1     | 0     | 0     | AD0 level |
 */
#define MPU6050_ADDRESS 0x68
//! @}

/**
 * @addtogroup CALI Calibration offset
 * @brief Data of calibration offset
 * @{
 */

//! @brief Calibrated X axis offset of Gyroscope
int GYRO_X_OFFSET;

//! @brief Calibrated Y axis offset of Gyroscope
int GYRO_Y_OFFSET;

//! @brief Calibrated Z axis offset of Gyroscope
int GYRO_Z_OFFSET;
/**
 * @}
 */

/**
 * @addtogroup APIs
 * @brief the functions and MACROs which will be used in application
 * @{
 */

//! @brief The register address of X axis of Accelerometer
#define ACCEL_X         ACCEL_XOUT_H

//! @brief The register address of Y axis of Accelerometer
#define ACCEL_Y         ACCEL_YOUT_H

//! @brief The register address of Z axis of Accelerometer
#define ACCEL_Z         ACCEL_ZOUT_H

//! @brief The register address of Temperature
#define TEMP            TEMP_OUT_H

//! @brief The register address of X axis of Gyroscope
#define GYRO_X          GYRO_XOUT_H

//! @brief The register address of Y axis of Gyroscope
#define GYRO_Y          GYRO_YOUT_H

//! @brief The register address of Z axis of Gyroscope
#define GYRO_Z          GYRO_ZOUT_H

void MPU6050_Init(void);
short MPU6050_ReadSensorData(unsigned char regAddress);
MPU6050Data MPU6050_ReadAllSensorData(void);
void IMU_Poll(IMU_DATA *data);
/**
 * @}
 */


#endif
//! end of __MPU_6050_H
